﻿using Abide_AddOn_API;
using Abide_AddOn_API.Classes;
using System;
using System.Drawing;
using System.Windows.Forms;

namespace Abide.Forms
{
    public partial class OptionsForm : Form
    {
        Form parent = null;
        public OptionsForm(Form Parent)
        {
            this.parent = Parent;
            InitializeComponent();
        }
        private void ColorPanelPicker_Click(object sender, EventArgs e)
        {
            Color SelectedColor = ((Panel)sender).BackColor;

            using (ColorDialog ClrDlg = new ColorDialog())
            {
                ClrDlg.Color = SelectedColor;

                if (ClrDlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    SelectedColor = ClrDlg.Color;
                    ((Panel)sender).BackColor = ClrDlg.Color;
                }
            }
        }
        private void OptionsForm_Load(object sender, EventArgs e)
        {
            //Initialize...

            //Load Themes
            foreach (ITheme Theme in Program.ThemeManager.Themes)
            {
                ThemeComboBox.Items.Add(Theme.Name);
            }

            //
            //UI
            //
            ThemeComboBox.SelectedItem = Settings.Theme;

            //
            //Visuals
            //
            ShowTVIconsCheckBox.Checked = Settings.UseTagViewIcons;

            //
            //Paths
            //
            H2PluginsBox.Text = Settings.H2_PluginsPath;
            H2MainMenuBox.Text = Settings.H2_MainMenuPath;
            H2SharedBox.Text = Settings.H2_SharedPath;
            H2SpSharedBox.Text = Settings.H2_SinglePlayerSharedPath;
            H2BPluginsBox.Text = Settings.H2B_PluginsPath;
            H2BMainMenuBox.Text = Settings.H2B_MainMenuPath;
            H2BSharedBox.Text = Settings.H2B_SharedPath;
        }
        private void ApplyButton_Click(object sender, EventArgs e)
        {
            //Check for Restart Changes
            bool RestartRequired = (Settings.Theme != ThemeComboBox.SelectedItem.ToString());

            //
            //UI
            //
            Settings.Theme = ThemeComboBox.SelectedItem.ToString();

            //
            //Visuals
            //
            Settings.UseTagViewIcons = ShowTVIconsCheckBox.Checked;

            //
            //Paths
            //
            Settings.H2_PluginsPath = H2PluginsBox.Text;
            Settings.H2_MainMenuPath = H2MainMenuBox.Text;
            Settings.H2_SharedPath = H2SharedBox.Text;
            Settings.H2_SinglePlayerSharedPath = H2SpSharedBox.Text;
            Settings.H2B_PluginsPath = H2BPluginsBox.Text;
            Settings.H2B_MainMenuPath = H2BMainMenuBox.Text;
            Settings.H2B_SharedPath = H2BSharedBox.Text;

            //Reload...
            parent.Refresh();

            //Show Message
            if (RestartRequired)
            {
                DialogResult DlgRes = MessageBox.Show(
                    "To fully apply these settings the application must be restarted. Would you like to restart now?",
                    "Restart?", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

                if (DlgRes == System.Windows.Forms.DialogResult.Yes)
                    Application.Restart();
            }
        }
        private void RestoreButton_Click(object sender, EventArgs e)
        {
            //Warn...
            if (MessageBox.Show("Warning: Doing this will reset all of Abide's settings.\r\nAre you sure you would like to continue?", 
                "Are you sure?", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == System.Windows.Forms.DialogResult.Yes)
            {
                //Reset...
                Settings.CreateAbideValues(true);
            }

            //Re-Initialize...
            OptionsForm_Load(this, e);
        }
        private void OpenThemesButton_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start(Settings.ThemesPath);
        }
        private void H2PluginPathBroswseButton_Click(object sender, EventArgs e)
        {
            using (FolderBrowserDialog FldDlg = new FolderBrowserDialog())
            {
                if (System.IO.Directory.Exists(Settings.H2_PluginsPath))
                {
                    FldDlg.SelectedPath = Settings.H2_PluginsPath;
                }

                if (FldDlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    H2PluginsBox.Text = FldDlg.SelectedPath;
                }
            }
        }
        private void H2MainMenuBrowseButton_Click(object sender, EventArgs e)
        {
            using (FileDialog OpenDlg = new OpenFileDialog())
            {
                OpenDlg.Title = "Select Mainmenu Map";
                OpenDlg.Filter = "Mainmenu Maps (mainmenu.map)|mainmenu.map";

                if (OpenDlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    H2MainMenuBox.Text = OpenDlg.FileName;
                }
            }
        }
        private void H2SharedBrowseButton_Click(object sender, EventArgs e)
        {
            using (FileDialog OpenDlg = new OpenFileDialog())
            {
                OpenDlg.Title = "Select Shared Map";
                OpenDlg.Filter = "Shared Maps (shared.map)|shared.map";

                if (OpenDlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    H2SharedBox.Text = OpenDlg.FileName;
                }
            }
        }
        private void H2SPSharedBrowseButton_Click(object sender, EventArgs e)
        {
            using (FileDialog OpenDlg = new OpenFileDialog())
            {
                OpenDlg.Title = "Select SP Shared Map";
                OpenDlg.Filter = "SP Shared Maps (single_player_shared.map)|single_player_shared.map";

                if (OpenDlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    H2SpSharedBox.Text = OpenDlg.FileName;
                }
            }
        }
        private void H2BMainMenuButton_Click(object sender, EventArgs e)
        {
            using (FileDialog OpenDlg = new OpenFileDialog())
            {
                OpenDlg.Title = "Select Mainmenu Map";
                OpenDlg.Filter = "Mainmenu Maps (mainmenu.map)|mainmenu.map";

                if (OpenDlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    H2BMainMenuBox.Text = OpenDlg.FileName;
                }
            }
        }
        private void H2BSharedButton_Click(object sender, EventArgs e)
        {
            using (FileDialog OpenDlg = new OpenFileDialog())
            {
                OpenDlg.Title = "Select Shared Map";
                OpenDlg.Filter = "Shared Maps (shared.map)|shared.map";

                if (OpenDlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    H2BSharedBox.Text = OpenDlg.FileName;
                }
            }
        }
        private void H2BPluginBrowseButton_Click(object sender, EventArgs e)
        {
            using (FolderBrowserDialog FldDlg = new FolderBrowserDialog())
            {
                if (System.IO.Directory.Exists(Settings.H2B_PluginsPath))
                {
                    FldDlg.SelectedPath = Settings.H2B_PluginsPath;
                }

                if (FldDlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    H2BPluginsBox.Text = FldDlg.SelectedPath;
                }
            }
        }
    }
}
