﻿using Abide.Controls;
using Abide_AddOn_API;
using Abide_AddOn_API.Classes;
using Abide_AddOn_API.Halo_2_Beta_Map;
using Abide_AddOn_API.Meta;
using Potential_Software.Win32;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

namespace Abide.UI_Elements
{
    public partial class Halo2BetaMapControl : UserControl
    {
        public Halo2BetaMap Map
        {
            get;
            protected set;
        }
        public Classes.AddOn_Manager AddOnManager
        {
            get;
            set;
        }
        ITool CurrentTool
        {
            get;
            set;
        }
        Dictionary<string, List<ITabPage>> ClassPages = new Dictionary<string, List<ITabPage>>();

        protected override void OnHandleCreated(EventArgs e)
        {
            base.OnHandleCreated(e);
            Uxtheme.SetWindowTheme(TagView.Handle, "explorer", null);
        }
        public static Icon Extract(string file, int number, bool largeIcon)
        {
            IntPtr large = IntPtr.Zero;
            IntPtr small = IntPtr.Zero;

            Potential_Software.Win32.Shell32.ExtractIconEx(file, number, out large, out small, 1);

            try
            {
                Icon ReturnValue = (Icon)(Icon.FromHandle(largeIcon ? large : small).Clone());
                User32.DestroyIcon(large);
                User32.DestroyIcon(small);
                return ReturnValue;
            }
            catch
            {
                return null;
            }
        }
        public Halo2BetaMapControl(string FileLocation)
        {
            InitializeComponent();
            TagView.ContextMenu = TagViewContextMenu;

            if (!Settings.UseTagViewIcons)
            {
                TagView.ImageList = null;
            }

            //Prepare Images...
            this.TagViewImgList.Images.Add(new Icon(Properties.Resources.Unknown_Map, 16, 16));
            this.TagViewImgList.Images.Add(Extract("shell32.dll", 155, false));
            this.TagViewImgList.Images.Add(Extract("shell32.dll", 70, false));
            this.TagViewImgList.Images.Add(Extract("shell32.dll", 302, false));
            this.TagViewImgList.Images.Add(Extract("shell32.dll", 116, false));
            this.TagViewImgList.Images.Add(Extract("shell32.dll", 80, false));

            /*
             * Folder Icon = 155
             * Text Icon = 70
             * Image Icon = 301 or 302
             * Sound Icon = 116
             * SBSP = 80
             */

            //Get H2Map
            Map = new Halo2BetaMap();
            Map.LoadMap(FileLocation);

            //Load AddOn Manager and AddOns
            AddOnManager = new Classes.AddOn_Manager(Map, Program.Xbox);

            //Load Tags...
            UpdateTagView();
        }

        private void MapControl_Load(object sender, EventArgs e)
        {
            //Load AddOns
            LoadAddOns();
        }
        private void TagView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (e.Node.Tag != null)
            {
                //Get Tag?
                int TagInt = int.Parse(e.Node.Tag.ToString());

                if (TagInt == -1)
                    PropertyView.SelectedObject = Map;
                else
                {
                    //Load Tag
                    PropertyView.SelectedObject = ByIdent(TagInt);
                    LoadTag(ByIdent(TagInt));
                }
            }
        }
        private void TagClassItem_Click(object sender, EventArgs e)
        {
            Settings.TagViewType_H2B = "TagType";
            TagPathItem.Checked = false;
            TagClassItem.Checked = true;
            UpdateTagView();
        }
        private void TagPathItem_Click(object sender, EventArgs e)
        {
            Settings.TagViewType_H2B = "TagPath";
            TagPathItem.Checked = true;
            TagClassItem.Checked = false;
            UpdateTagView();
        }
        private void LoadTag(Abide_AddOn_API.Halo_2_Beta_Map.IndexEntry Entry)
        {
            AddOnManager.Plugin = new HaloPlugin(ParsePluginPath(Entry.Class));
            AddOnManager.SelectedEntry = new Abide_AddOn_API.IndexEntry();
            AddOnManager.SelectedEntry.H2B_Entry = Entry;

            if (CurrentTool != null)
                AddOnManager.UpdateTag(CurrentTool);

            if (ClassPages.ContainsKey("*"))
                foreach (ITabPage IPage in ClassPages["*"])
                {
                    HandlePropertiesTab(IPage);
                }
            if (ClassPages.ContainsKey(Entry.Class.ToLower()))
                foreach (ITabPage IPage in ClassPages[Entry.Class.ToLower()])
                {
                    HandlePropertiesTab(IPage);
                }

            for (int i = 0; i < PropertyTabView.TabCount; i++)
            {
                if (PropertyTabView.TabPages[i] is AddOnTabPage)
                {
                    AddOnTabPage AddonPage = (AddOnTabPage)PropertyTabView.TabPages[i];

                    if (AddonPage != null)
                    {
                        List<string> Filter = new List<string>(AddonPage.EnabledTypes);
                        if (!Filter.Contains("*") & !Filter.Contains(Entry.Class.ToLower()))
                        {
                            PropertyTabView.TabPages.RemoveAt(i);
                        }
                    }
                }
            }
        }
        private void LoadAddOnItem_Click(object sender, EventArgs e)
        {
            using (FileDialog OpenDlg = new OpenFileDialog())
            {
                OpenDlg.Title = "Open AddOn Assembly";
                OpenDlg.Filter = "Abide AddOn Assemblies (*.dll)|*.dll";

                if (OpenDlg.ShowDialog() == DialogResult.OK)
                {
                    try
                    {
                        //Add (possible) AddOns.
                        AddOnManager.AddTools(OpenDlg.FileName);
                        AddOnManager.AddTabPages(OpenDlg.FileName);
                        AddOnManager.AddMenuButtons(OpenDlg.FileName);
                    }
                    catch (Exception ex)
                    {
                        Console.WriteLine(ex.StackTrace);
                    }
                    finally { ApplyAddOns(); }
                }
            }
        }
        private void OpenFolderItem_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start(Settings.AddOnsPath);
        }
        private string ParsePluginPath(string TagClass)
        {
            return Settings.H2B_PluginsPath + "\\" + TagClass.Replace("<", "_").Replace(">", "_").Replace(" ", string.Empty) + ".ent";
        }
        private void UpdateTagView()
        {
            //Begin Update...
            TagView.BeginUpdate();

            //Clear
            TagView.Nodes.Clear();

            //Create...
            TreeNode MapNode = new TreeNode(Map.MapName);
            MapNode.Tag = -1;

            //Get TagView Type
            if (Settings.TagViewType_H2B == "TagType")
            {
                //Add Nodes...
                MapNode.Nodes.AddRange(Map.GetTreeViewTagTypes());
                TagPathItem.Checked = false;
                TagClassItem.Checked = true;

            }
            else if (Settings.TagViewType_H2B == "TagPath")
            {
                //Add Nodes...
                MapNode.Nodes.AddRange(Map.GetTreeViewFolder());
                TagPathItem.Checked = true;
                TagClassItem.Checked = false;
            }
            else
            {
                //Show Message
                MessageBox.Show("We don't know how to disply your tags, so we're going to default to Tag Type display type.",
                    "Whoops!", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                //Add Nodes...
                MapNode.Nodes.AddRange(Map.GetTreeViewTagTypes());
                TagPathItem.Checked = false;
                TagClassItem.Checked = true;
            }

            //Prepare
            MapNode.Expand();
            TagView.Nodes.Add(MapNode);

            //Sort
            TagView.Sort();

            //End Update...
            TagView.EndUpdate();
        }
        private void LoadAddOns()
        {
            try
            {
                //Get AddOn Directory
                string AddOnsFolder = Settings.AddOnsPath;

                //Create?
                if (!Directory.Exists(AddOnsFolder))
                    Directory.CreateDirectory(AddOnsFolder);

                //Get AddOn Folders...
                string[] AddOnFolders = Directory.GetDirectories(AddOnsFolder);

                //Loop...
                foreach (string Folder in AddOnFolders)
                {
                    string[] Files = Directory.GetFiles(Folder);

                    foreach (string File in Files)
                    {
                        //Add (possible) AddOns.
                        AddOnManager.AddTools(File);
                        AddOnManager.AddTabPages(File);
                        AddOnManager.AddMenuButtons(File);
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.StackTrace);
            }
            finally { ApplyAddOns(); }
        }
        private void ApplyAddOns()
        {
            #region Clear
            //MetaToolStrip
            MetaToolStrip.Items.Clear();
            MetaToolStrip.Items.Add(ToolsDropdownButton);
            MetaToolStrip.Items.Add(MapLabel);
            //ToolsDropdownButton
            ToolsDropdownButton.DropDownItems.Clear();
            ToolsDropdownButton.DropDownItems.Add(AddOnsItem);
            ToolsDropdownButton.DropDownItems.Add(ToolSeparator);
            //ClassPages
            ClassPages.Clear();
            #endregion
            #region Handle ITools
            //Show Separator?
            ToolSeparator.Visible = AddOnManager.AddOnCount > 0;

            //Create Item...
            ToolStripMenuItem ToolItem = null;

            //Add AddOns into Tools
            foreach (ITool Tool in AddOnManager.Tools)
            {
                //Prepare...
                ToolItem = new ToolStripMenuItem();
                ToolItem.Text = Tool.Name;
                ToolItem.Tag = AddOnManager.Tools.IndexOf(Tool);
                ToolItem.Click += ToolItem_Click;
                ToolItem.ToolTipText = string.Format("{0}\r\n{1}\r\nv{2}\r\nBy: {3}",
                    Tool.Name, Tool.Description, Tool.Version, Tool.Author);

                //Add...
                ToolsDropdownButton.DropDownItems.Add(ToolItem);
            }
            #endregion
            #region Handle ITabPages
            foreach (ITabPage Page in AddOnManager.TabPages)
            {
                string[] Classes = Page.TagFilter.Split('|');
                foreach (string Class in Classes)
                {
                    //Initialize
                    Page.Initialize();

                    //Check...
                    if (!ClassPages.ContainsKey(Class))
                        ClassPages.Add(Class, new List<ITabPage>());

                    //Add
                    ClassPages[Class].Add(Page);
                }
            }
            #endregion
            #region Handle IMenuButtons
            //Add AddOns into Tools
            foreach (IMenuButton MenuButton in AddOnManager.MenuButtons)
            {
                if (MenuButton.ChildItems != null && MenuButton.ChildItems.Count > 0)
                {
                    //Prepare
                    ToolStripDropDownButton MenuButtonItem = new ToolStripDropDownButton(MenuButton.Text);
                    MenuButtonItem.Tag = MenuButton;
                    MenuButtonItem.Click += MenuButtonItem_Click;
                    MenuButtonItem.DropDownItems.AddRange(MenuButton.ChildItems.ToArray());

                    //Add
                    MetaToolStrip.Items.Add(MenuButtonItem);
                }
                else
                {
                    //Prepare
                    ToolStripButton MenuButtonItem = new ToolStripButton(MenuButton.Text);
                    MenuButtonItem.Tag = MenuButton;
                    MenuButtonItem.Click += MenuButtonItem_Click;

                    //Add
                    MetaToolStrip.Items.Add(MenuButtonItem);
                }
            }
            #endregion

            //Change Labels
            TabPageLabel.Text = string.Format("Tab Page Count: {0}", AddOnManager.TabPageCount);
            ToolCountLabel.Text = string.Format("Tool Count: {0}", AddOnManager.AddOnCount);
            MenuItemsLabel.Text = string.Format("Menu Button Count: {0}", AddOnManager.MenuButtonCount);
        }
        private void HandlePropertiesTab(ITabPage IPage)
        {
            //Update
            AddOnManager.UpdateTag(IPage);

            AddOnTabPage Page = PropertyTabView.TabPages[IPage.Text] as AddOnTabPage;
            if (Page == null)
            {
                Page = new AddOnTabPage(IPage.Text);
                Page.EnabledTypes = IPage.TagFilter.Split('|');
                Page.Name = IPage.Text;
                Control UI = IPage.Interface;
                UI.Dock = DockStyle.Fill;

                Page.Controls.Add(UI);
                PropertyTabView.TabPages.Add(Page);
                PropertyTabView.SelectedIndex = PropertyTabView.TabCount - 1;
            }
        }
        private Abide_AddOn_API.Halo_2_Beta_Map.IndexEntry ByIdent(int Ident)
        {
            for (int i = 0; i < Map.IndexEntries.Length; i++)
            {
                if (Map.IndexEntries[i].Ident == Ident)
                    return Map.IndexEntries[i];
            }

            return null;
        }

        void ToolItem_Click(object sender, EventArgs e)
        {
            //Check if same AddOn was selected...
            int SelectedIndex = (int)((ToolStripMenuItem)sender).Tag;

            if (CurrentTool != AddOnManager.Tools[SelectedIndex])
            {

                //Destroy Previous AddOn
                if (CurrentTool != null)
                    CurrentTool.Dispose();

                //Create New
                CurrentTool = AddOnManager.Tools[SelectedIndex];
                CurrentTool.Initialize();
                Control UI = CurrentTool.Interface;
                UI.Dock = DockStyle.Fill;
                MetaPanel.Controls.Clear();
                MetaPanel.Controls.Add(UI);
            }
        }
        void MenuButtonItem_Click(object sender, EventArgs e)
        {
            if (sender is ToolStripItem)
            {
                ToolStripItem Sender = (ToolStripItem)sender;
                if (Sender.Tag is IMenuButton)
                {
                    IMenuButton MenuButton = (IMenuButton)Sender.Tag;
                    MenuButton.OnMenuButtonClicked();
                }
            }
        }
    }
}
