﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;

namespace AbideHaloLibrary.Classes
{
    public static class Settings
    {
        #region Properties
        public static string PluginsPath
        {
            get { return Settings.GetKey(@"Xbox\Halo2\Paths", "Plugins").ToString(); }
            set { Settings.SetKey(@"Xbox\Halo2\Paths", "Plugins", value, true); if (PluginsPathChanged != null) PluginsPathChanged.Invoke(value, null); }
        }
        public static string ModdedMapsPath
        {
            get { return Settings.GetKey(@"Xbox\Halo2\Paths", "Maps").ToString(); }

            set { Settings.SetKey(@"Xbox\Halo2\Paths", "Maps", value, true); }
        }
        public static string CleanMapsPath
        {
            get { return Settings.GetKey(@"Xbox\Halo2\Paths", "CleanMaps").ToString(); }

            set { Settings.SetKey(@"Xbox\Halo2\Paths", "CleanMaps", value, true); }
        }
        public static string ExtractFolderPath
        {
            get { return Settings.GetKey(@"Xbox\Halo2\Paths", "Extracts").ToString(); }

            set { Settings.SetKey(@"Xbox\Halo2\Paths", "Extracts", value, true); }
        }
        public static string MainMenuPath
        {
            get { return Settings.GetKey(@"Xbox\Halo2\Paths", "MainMenu").ToString(); }

            set { Settings.SetKey(@"Xbox\Halo2\Paths", "MainMenu", value, true); }
        }
        public static string SharedPath
        {
            get { return Settings.GetKey(@"Xbox\Halo2\Paths", "Shared").ToString(); }

            set { Settings.SetKey(@"Xbox\Halo2\Paths", "Shared", value, true); }
        }
        public static string SinglePlayerSharedPath
        {
            get { return Settings.GetKey(@"Xbox\Halo2\Paths", "SPShared").ToString(); }

            set { Settings.SetKey(@"Xbox\Halo2\Paths", "SPShared", value, true); }
        }
        public static string AddOnsPath
        {
            get { return Settings.GetKey(@"Xbox\Halo2\Paths", "AddOns").ToString(); }

            set { Settings.SetKey(@"Xbox\Halo2\Paths", "AddOns", value, true); if (PluginsPathChanged != null) PluginsPathChanged.Invoke(value, null); }
        }
        public static string DebugIp
        {
            get { return Settings.GetKey(@"Xbox", "Debug Ip").ToString(); }

            set { Settings.SetKey(@"Xbox", "Debug Ip", value, true); if (DebugIpChanged != null) DebugIpChanged.Invoke(value, null); }
        }
        public static string TagViewType
        {
            get { return Settings.GetKey(@"Xbox\Halo2\Abide", "TagViewType").ToString(); }

            set { Settings.SetKey(@"Xbox\Halo2\Abide", "TagViewType", value, true); }
        }
        public static string FilterValue
        {
            get { return Settings.GetKey(@"Xbox\Halo2\Abide", "Filter").ToString(); }

            set { Settings.SetKey(@"Xbox\Halo2\Abide", "Filter", value, true); }
        }
        public static string LastTool
        {
            get { return Settings.GetKey(@"Xbox\Halo2\Abide", "LastTool").ToString(); }

            set { Settings.SetKey(@"Xbox\Halo2\Abide", "LastTool", value, true); }
        }
        public static string UseSwapperForm
        {
            get { return Settings.GetKey(@"Xbox\Halo2\Abide", "UseSwapperForm").ToString(); }

            set { Settings.SetKey(@"Xbox\Halo2\Abide", "UseSwapperForm", value, true); }
        }
        public static string EnablePokeHistory
        {
            get { return Settings.GetKey(@"Xbox\Halo2\Abide", "EnablePokeHistory").ToString(); }

            set { Settings.SetKey(@"Xbox\Halo2\Abide", "EnablePokeHistory", value, true); }
        }
        public static string AutoConnect
        {
            get { return Settings.GetKey(@"Xbox\Halo2\Abide", "AutoConnect").ToString(); }

            set { Settings.SetKey(@"Xbox\Halo2\Abide", "AutoConnect", value, true); }
        }
        public static List<string> RecentMaps
        {
            get
            {
                //Create Dictionary to Return Values
                List<string> ReturnValue = new List<string>();

                //Open Software Key
                RegistryKey Software = Registry.CurrentUser.OpenSubKey("Software", false);

                if (Software != null)
                {
                    //Open Values Key
                    RegistryKey Pathskey = Software.OpenSubKey(@"Xbox\Halo2\Recent Files", false);

                    if (Pathskey != null)
                    {
                        //Get registered values
                        string[] RegisteredPaths = Pathskey.GetValueNames();

                        //Loop through values
                        foreach (string valuename in RegisteredPaths)
                        {
                            //Get Value object
                            string Path = GetKey(@"Xbox\Halo2\Recent Files", valuename).ToString();

                            //Add Value To Dictionary
                            ReturnValue.Add(Path);
                        }

                        //Close
                        Pathskey.Close();
                    }

                    //Close Keys
                    Software.Close();
                }

                return ReturnValue;
            }
            set
            {
                CreateRecentMapsKey();

                //Open Software Key
                RegistryKey Software = Registry.CurrentUser.OpenSubKey("Software", true);

                //Delete Key
                Software.DeleteSubKey(@"Xbox\Halo2\Recent Files");

                //Open Values Key
                RegistryKey Valueskey = Software.CreateSubKey(@"Xbox\Halo2\Recent Files");

                //Save Values Into Registry
                for (int i = 0; i < Math.Min(value.Count, 10); i++)
                {
                    SetKey(@"Xbox\Halo2\Recent Files", i.ToString(), value[i], true);
                }

                //Close Values
                Valueskey.Close();
                Software.Close();
            }
        }

        public static bool UseTagViewIcons
        {
            get { return (GetKey(@"Xbox\Halo2\Abide", "UseTVIcons").ToString().ToLower() == "true"); }
            set { SetKey(@"Xbox\Halo2\Abide", "UseTVIcons", value.ToString().ToLower(), true); }
        }
        public static string Theme
        {
            get { return GetKey(@"Xbox\Halo2\Abide", "Theme").ToString(); }
            set { SetKey(@"Xbox\Halo2\Abide", "Theme", value.ToString(), true); }
        }
        public static bool UseCustomColors
        {
            get { return (GetKey(@"Xbox\Halo2\Abide", "UseCustomColors").ToString().ToLower() == "true"); }
            set { SetKey(@"Xbox\Halo2\Abide", "UseCustomColors", value.ToString().ToLower(), true); }
        }

        public static Color Active
        {
            get { return Color.FromArgb((int)GetKey(@"Xbox\Halo2\Abide", "TabColor1")); }
            set { SetKey(@"Xbox\Halo2\Abide", "TabColor1", value.ToArgb(), true); }
        }
        public static Color ActiveGradient
        {
            get { return Color.FromArgb((int)GetKey(@"Xbox\Halo2\Abide", "TabColor2")); }
            set { SetKey(@"Xbox\Halo2\Abide", "TabColor2", value.ToArgb(), true); }
        }
        public static Color Inactive
        {
            get { return Color.FromArgb((int)GetKey(@"Xbox\Halo2\Abide", "TabColor3")); }
            set { SetKey(@"Xbox\Halo2\Abide", "TabColor3", value.ToArgb(), true); }
        }
        public static Color InactiveGradient
        {
            get { return Color.FromArgb((int)GetKey(@"Xbox\Halo2\Abide", "TabColor4")); }
            set { SetKey(@"Xbox\Halo2\Abide", "TabColor4", value.ToArgb(), true); }
        }
        public static Color Selected
        {
            get { return Color.FromArgb((int)GetKey(@"Xbox\Halo2\Abide", "TabColor5")); }
            set { SetKey(@"Xbox\Halo2\Abide", "TabColor5", value.ToArgb(), true); }
        }
        public static Color SelectedGradient
        {
            get { return Color.FromArgb((int)GetKey(@"Xbox\Halo2\Abide", "TabColor6")); }
            set { SetKey(@"Xbox\Halo2\Abide", "TabColor6", value.ToArgb(), true); }
        }
        public static Color ActiveText
        {
            get { return Color.FromArgb((int)GetKey(@"Xbox\Halo2\Abide", "TabColor7")); }
            set { SetKey(@"Xbox\Halo2\Abide", "TabColor7", value.ToArgb(), true); }
        }
        public static Color InactiveText
        {
            get { return Color.FromArgb((int)GetKey(@"Xbox\Halo2\Abide", "TabColor8")); }
            set { SetKey(@"Xbox\Halo2\Abide", "TabColor8", value.ToArgb(), true); }
        }
        public static Color Border
        {
            get { return Color.FromArgb((int)GetKey(@"Xbox\Halo2\Abide", "TabColor9")); }
            set { SetKey(@"Xbox\Halo2\Abide", "TabColor9", value.ToArgb(), true); }
        }
        #endregion

        #region Event Handlers
        public static event EventHandler ValueVisibilityChanged;
        public static event EventHandler DebugIpChanged;
        public static event EventHandler PluginsPathChanged;
        #endregion

        #region Methods
        public static void CreateKeys()
        {
            CreateSharedKeys();
            CreateAbideValues();
        }
        public static void CreateAbideValues()
        {
            //Tag Tree View Type Tags,Folders,Filtered
            SetKey(@"Xbox\Halo2\Abide", "TagViewType", "TagType", false);

            //FilterBox Value
            SetKey(@"Xbox\Halo2\Abide", "Filter", "Tag Filter", false);

            //Current Tool, Reflexive, Ident, Meta
            SetKey(@"Xbox\Halo2\Abide", "LastTool", "Reflexive", false);

            //Use Swapper Form True, False
            SetKey(@"Xbox\Halo2\Abide", "UseSwapperForm", "false", false);

            //Use Poke History
            SetKey(@"Xbox\Halo2\Abide", "EnablePokeHistory", "false", false);

            //Auto Connect
            SetKey(@"Xbox\Halo2\Abide", "AutoConnect", "false", false);

            //Path Values
            SetKey(@"Xbox 360\Halo Reach\Paths", "Plugins", "Not Set", false);
            SetKey(@"Xbox 360\Halo Reach\Paths", "TagLists", "Not Set", false);
            SetKey(@"Xbox 360\Halo Reach\Paths", "StringLists", "Not Set", false);

            //TreeView Icons
            SetKey(@"Xbox\Halo2\Abide", "UseTVIcons", "true", false);

            //Default Theme
            SetKey(@"Xbox\Halo2\Abide", "Theme", "Default", false);

            //Use Tab Colors
            SetKey(@"Xbox\Halo2\Abide", "UseCustomColors", "false", false);

            //Use CP theme
            SetKey(@"Xbox\Halo2\Abide", "UseCaptainPoopfaceTheme", "false", false);

            //Tab Colors
            SetKey(@"Xbox\Halo2\Abide", "TabColor1", int.Parse("ffffffff", System.Globalization.NumberStyles.HexNumber), false);
            SetKey(@"Xbox\Halo2\Abide", "TabColor2", int.Parse("fff0f0f0", System.Globalization.NumberStyles.HexNumber), false);
            SetKey(@"Xbox\Halo2\Abide", "TabColor3", int.Parse("fff0f0f0", System.Globalization.NumberStyles.HexNumber), false);
            SetKey(@"Xbox\Halo2\Abide", "TabColor4", int.Parse("ffe3e3e3", System.Globalization.NumberStyles.HexNumber), false);
            SetKey(@"Xbox\Halo2\Abide", "TabColor5", int.Parse("ffffffff", System.Globalization.NumberStyles.HexNumber), false);
            SetKey(@"Xbox\Halo2\Abide", "TabColor6", int.Parse("ffe3e3e3", System.Globalization.NumberStyles.HexNumber), false);
            SetKey(@"Xbox\Halo2\Abide", "TabColor7", int.Parse("ff000000", System.Globalization.NumberStyles.HexNumber), false);
            SetKey(@"Xbox\Halo2\Abide", "TabColor8", int.Parse("ff484848", System.Globalization.NumberStyles.HexNumber), false);
            SetKey(@"Xbox\Halo2\Abide", "TabColor9", int.Parse("ff808080", System.Globalization.NumberStyles.HexNumber), false);
        }
        public static void CreateAbideValues(bool Overwrite)
        {
            //Tag Tree View Type Tags,Folders,Filtered
            SetKey(@"Xbox\Halo2\Abide", "TagViewType", "TagType", Overwrite);

            //FilterBox Value
            SetKey(@"Xbox\Halo2\Abide", "Filter", "Tag Filter", Overwrite);

            //Current Tool, Reflexive, Ident, Meta
            SetKey(@"Xbox\Halo2\Abide", "LastTool", "Reflexive", Overwrite);

            //Use Swapper Form True, False
            SetKey(@"Xbox\Halo2\Abide", "UseSwapperForm", "false", Overwrite);

            //Use Poke History
            SetKey(@"Xbox\Halo2\Abide", "EnablePokeHistory", "false", Overwrite);

            //Auto Connect
            SetKey(@"Xbox\Halo2\Abide", "AutoConnect", "false", Overwrite);

            //Path Values
            SetKey(@"Xbox 360\Halo Reach\Paths", "Plugins", "Not Set", Overwrite);
            SetKey(@"Xbox 360\Halo Reach\Paths", "TagLists", "Not Set", Overwrite);
            SetKey(@"Xbox 360\Halo Reach\Paths", "StringLists", "Not Set", Overwrite);

            //TreeView Icons
            SetKey(@"Xbox\Halo2\Abide", "UseTVIcons", "true", Overwrite);

            //Default Theme
            SetKey(@"Xbox\Halo2\Abide", "Theme", "Default", Overwrite);

            //Use CP theme
            SetKey(@"Xbox\Halo2\Abide", "UseCaptainPoopfaceTheme", "false", Overwrite);

            //Use Tab Colors
            SetKey(@"Xbox\Halo2\Abide", "UseCustomColors", "false", Overwrite);

            //Tab Colors
            SetKey(@"Xbox\Halo2\Abide", "TabColor1", int.Parse("ffffffff", System.Globalization.NumberStyles.HexNumber), Overwrite);
            SetKey(@"Xbox\Halo2\Abide", "TabColor2", int.Parse("fff0f0f0", System.Globalization.NumberStyles.HexNumber), Overwrite);
            SetKey(@"Xbox\Halo2\Abide", "TabColor3", int.Parse("fff0f0f0", System.Globalization.NumberStyles.HexNumber), Overwrite);
            SetKey(@"Xbox\Halo2\Abide", "TabColor4", int.Parse("ffe3e3e3", System.Globalization.NumberStyles.HexNumber), Overwrite);
            SetKey(@"Xbox\Halo2\Abide", "TabColor5", int.Parse("ffffffff", System.Globalization.NumberStyles.HexNumber), Overwrite);
            SetKey(@"Xbox\Halo2\Abide", "TabColor6", int.Parse("ffe3e3e3", System.Globalization.NumberStyles.HexNumber), Overwrite);
            SetKey(@"Xbox\Halo2\Abide", "TabColor7", int.Parse("ff000000", System.Globalization.NumberStyles.HexNumber), Overwrite);
            SetKey(@"Xbox\Halo2\Abide", "TabColor8", int.Parse("ff484848", System.Globalization.NumberStyles.HexNumber), Overwrite);
            SetKey(@"Xbox\Halo2\Abide", "TabColor9", int.Parse("ff808080", System.Globalization.NumberStyles.HexNumber), Overwrite);
        }
        public static void CreateSharedKeys()
        {
            //Software Key
            RegistryKey SoftwareKey = Registry.CurrentUser.OpenSubKey(@"Software", true);

            //Write Recent Maps Key
            CreateRecentMapsKey();

            //Debug Ip
            CreateDebugIpValue();

            //Path Values
            SetKey(@"Xbox\Halo2\Paths", "Plugins", "Not Set", false);
            SetKey(@"Xbox\Halo2\Paths", "Maps", "Not Set", false);
            SetKey(@"Xbox\Halo2\Paths", "CleanMaps", "Not Set", false);
            SetKey(@"Xbox\Halo2\Paths", "Extracts", "Not Set", false);
            SetKey(@"Xbox\Halo2\Paths", "MainMenu", "Not Set", false);
            SetKey(@"Xbox\Halo2\Paths", "Shared", "Not Set", false);
            SetKey(@"Xbox\Halo2\Paths", "SPShared", "Not Set", false);
            SetKey(@"Xbox\Halo2\Paths", "AddOns", "Not Set", false);
        }
        public static void CreateSharedKeys(bool Overwrite)
        {
            //Software Key
            RegistryKey SoftwareKey = Registry.CurrentUser.OpenSubKey(@"Software", true);

            //Write Recent Maps Key
            CreateRecentMapsKey();

            //Debug Ip
            CreateDebugIpValue();

            //Path Values
            SetKey(@"Xbox\Halo2\Paths", "Plugins", "Not Set", Overwrite);
            SetKey(@"Xbox\Halo2\Paths", "Maps", "Not Set", Overwrite);
            SetKey(@"Xbox\Halo2\Paths", "CleanMaps", "Not Set", Overwrite);
            SetKey(@"Xbox\Halo2\Paths", "Extracts", "Not Set", Overwrite);
            SetKey(@"Xbox\Halo2\Paths", "MainMenu", "Not Set", Overwrite);
            SetKey(@"Xbox\Halo2\Paths", "Shared", "Not Set", Overwrite);
            SetKey(@"Xbox\Halo2\Paths", "SPShared", "Not Set", Overwrite);
            SetKey(@"Xbox\Halo2\Paths", "AddOns", "Not Set", Overwrite);
        }
        private static void CreateRecentMapsKey()
        {
            //Software Key
            RegistryKey SoftwareKey = Registry.CurrentUser.OpenSubKey(@"Software", true);

            //Open recent Maps
            RegistryKey RecentFiles = SoftwareKey.OpenSubKey(@"Xbox\Halo2\Recent Files", true);

            if (RecentFiles == null)
            {
                //Create New Key
                RecentFiles = SoftwareKey.CreateSubKey(@"Xbox\Halo2\Recent Files");
            }
        }
        public static void CreateDebugIpValue()
        {
            //Ip
            SetKey("Xbox", "Debug Ip", "<Auto>", false);

            SetKey("Xbox 360", "Debug Ip", "0.0.0.0", false);
        }
        public static void ChangeVisibleValue(string ValueName, bool Value)
        {
            //Change Value
            SetKey(@"Xbox\Halo2\Abide\Visible Values", ValueName, Value, true);

            //invoke Event
            if (ValueVisibilityChanged != null)
                ValueVisibilityChanged.Invoke(ValueName, null);
        }
        public static object GetKey(string key, string valuename)
        {
            object KeyValue = null;

            //Get Software Key
            RegistryKey SoftwareKey = Registry.CurrentUser.OpenSubKey(@"Software", true);

            //Get Key
            RegistryKey Key = SoftwareKey.OpenSubKey(key, true);

            //Check if Key is Null
            if (Key == null)
            {
                //Create New Key
                Key = SoftwareKey.CreateSubKey(key);
            }
            else
            {
                //Get Key Value
                KeyValue = Key.GetValue(valuename);
            }

            //Close Keys
            Key.Close();

            SoftwareKey.Close();

            return KeyValue;
        }
        public static void SetKey(string key, string valuename, object value, bool overwrite)
        {
            //Create Software Key
            RegistryKey SoftwareKey = Registry.CurrentUser.CreateSubKey(@"Software", RegistryKeyPermissionCheck.Default);

            //Open Key
            RegistryKey Key = SoftwareKey.CreateSubKey(key, RegistryKeyPermissionCheck.Default);

            //Check if Key is Null
            if (Key == null)
            {
                //Create New Key
                Key = SoftwareKey.CreateSubKey(key);
            }

            //Get Value Names
            string[] Keys = Key.GetValueNames();

            //Overwrite if value exists and overwrite is true
            if (Keys.Contains(valuename) && overwrite)
                Key.SetValue(valuename, value);

            //Create the value if it doesn't exist
            if (!Keys.Contains(valuename))
                Key.SetValue(valuename, value);


            //Close Keys
            Key.Close();

            SoftwareKey.Close();
        }
        #endregion
    }
}
