﻿using AbideHaloLibrary.Map;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace AbideHaloLibrary.Halo_2_Map
{
    public class IndexTableHeader
    {
        public int PrimaryMagic
        {
            get { return primaryMagicConstant - indexHeaderOffset + 32; }
        }
        public int SecondaryMagic
        {
            get { return matgOffset - metaStart; }
        }
        public int TagListCount
        {
            get { return tagListCount; }
        }
        public int ObjectIndexOffset
        {
            get { return objectIndexOffset; }
        }
        public int ScenarioID
        {
            get { return scenarioID; }
        }
        public int TagIDStart
        {
            get { return tagIDStart; }
        }
        public int ObjectCount
        {
            get { return objectCount; }
        }
        public int ObjectStart
        {
            get { return objectStart; }
        }

        int indexHeaderOffset;
        int objectStart;
        int matgOffset;
        int metaStart;

        int primaryMagicConstant;
        int tagListCount;
        int objectIndexOffset;
        int scenarioID;
        int tagIDStart;
        int unknown;
        int objectCount;
        char[] tags;

        public IndexTableHeader(int IndexTableHeaderOffset, int MetaStart, HaloIO IO)
        {
            metaStart = MetaStart;
            indexHeaderOffset = IndexTableHeaderOffset;

            IO.MapStream.Position = IndexTableHeaderOffset;
            primaryMagicConstant = IO.In.ReadInt32();
            tagListCount = IO.In.ReadInt32();
            objectIndexOffset = IO.In.ReadInt32();
            scenarioID = IO.In.ReadInt32();
            tagIDStart = IO.In.ReadInt32();
            unknown = IO.In.ReadInt32();
            objectCount = IO.In.ReadInt32();
            tags = IO.In.ReadChars(4);

            //Determine Secondary Magic...
            objectStart = (IndexTableHeaderOffset + 32) + (12 * tagListCount);
            IO.MapStream.Position = objectStart + 8;
            matgOffset = IO.In.ReadInt32();
        }
    }

    public class TagListEntry
    {
        public string Class
        {
            get
            {
                string ret = string.Empty;

                for (int i = 0; i < objectClass.Length; i++)
                {
                    if (objectClass[i] != '�')
                        ret += objectClass[i];
                }

                char[] Class = ret.ToCharArray();
                Array.Reverse(Class);
                return new string(Class);
            }
        }
        public string ParentClass
        {
            get
            {
                string ret = string.Empty;

                for (int i = 0; i < parentClass.Length; i++)
                {
                    if (parentClass[i] != '�')
                        ret += parentClass[i];
                }

                char[] Class = ret.ToCharArray();
                Array.Reverse(Class);
                return new string(Class);
            }
        }
        public string GrandparentClass
        {
            get
            {
                string ret = string.Empty;

                for (int i = 0; i < grandparentClass.Length; i++)
                {
                    if (grandparentClass[i] != '�')
                        ret += grandparentClass[i];
                }

                char[] Class = ret.ToCharArray();
                Array.Reverse(Class);
                return new string(Class);
            }
        }

        char[] objectClass;
        char[] parentClass;
        char[] grandparentClass;

        public TagListEntry(int Offset, HaloIO IO)
        {
            IO.MapStream.Position = Offset;
            objectClass = IO.In.ReadChars(4);
            parentClass = IO.In.ReadChars(4);
            grandparentClass = IO.In.ReadChars(4);
        }
        public override string ToString()
        {
            return string.Format("[{0}]-[{1}]-[{2}]", Class, ParentClass, GrandparentClass);
        }
    }

    public class ObjectEntry
    {
        public string Tag
        {
            get
            {
                string ret = string.Empty;

                for (int i = 0; i < tag.Length; i++)
                {
                    if (tag[i] != '\0')
                        ret += tag[i];
                }

                char[] Class = ret.ToCharArray();
                Array.Reverse(Class);
                return new string(Class);
            }
        }
        public int ID
        {
            get { return id; }
        }
        public int Offset
        {
            get { return offset - secondaryMagic; }
        }
        public int RawOffset
        {
            get { return offset; }
        }
        public int Size
        {
            get { return metaSize; }
        }

        int secondaryMagic;

        char[] tag;
        int id;
        int offset;
        int metaSize;

        public ObjectEntry(int Offset, int SecondaryMagic, HaloIO IO)
        {
            secondaryMagic = SecondaryMagic;
            IO.MapStream.Position = Offset;
            tag = IO.In.ReadChars(4);
            id = IO.In.ReadInt32();
            offset = IO.In.ReadInt32();
            metaSize = IO.In.ReadInt32();
        }
        public override string ToString()
        {
            return string.Format("{0} ID: {1} Size: {2}", Tag, id, metaSize);
        }
    }

    public class IndexEntry
    {
        public string FileName
        {
            get { return fileName; }
        }
        public string Class
        {
            get { return new string(tag); }
        }
        public int Offset
        {
            get { return offset; }
        }
        public int VirtualOffset
        {
            get { return virtualOffset; }
        }
        public int Ident
        {
            get { return id; }
        }
        public int Size
        {
            get { return size; }
        }

        string fileName;
        char[] tag;
        int virtualOffset;
        int offset;
        int id;
        int size;

        public IndexEntry(ObjectEntry obj, string Name)
        {
            fileName = Name;
            tag = obj.Tag.ToCharArray();
            virtualOffset = obj.RawOffset;
            offset = obj.Offset;
            id = obj.ID;
            size = obj.Size;
        }
        public override string ToString()
        {
            return string.Format("{0}: {1}", Class, FileName);
        }
        public void SetVirtualOffset(int Value)
        {
            virtualOffset = Value;
        }
        public void SetSize(int Value)
        {
            size = Value;
        }

        public class IndexEntryCollection : IList, ICollection, IEnumerable
        {
            ArrayList InnerList;

            public IndexEntryCollection()
            {
                this.InnerList = new ArrayList(new List<IndexEntry>());
            }

            bool IList.IsReadOnly
            {
                get { return InnerList.IsReadOnly; }
            }
            public bool IsReadOnly
            {
                get { return InnerList.IsReadOnly; }
            }

            object IList.this[int index]
            {
                get { return InnerList[index]; }
                set { InnerList[index] = value; }
            }
            public IndexEntry this[int index_or_ident]
            {
                get
                {
                    if (index_or_ident > -1 && index_or_ident < InnerList.Count)
                    {
                        return InnerList[index_or_ident] as IndexEntry;
                    }
                    else
                    {
                        foreach (IndexEntry Entry in InnerList)
                        {
                            if (Entry.id == index_or_ident)
                            {
                                return Entry;
                            }
                        }
                    }

                    return null;
                }
                set
                {
                    if (index_or_ident > -1 && index_or_ident < InnerList.Count)
                    {
                        InnerList[index_or_ident] = value;
                    }
                    else
                    {
                        for (int i = 0; i < InnerList.Count; i++)
                        {
                            if (InnerList[i].GetType() == typeof(IndexEntry))
                            {
                                IndexEntry Entry = (IndexEntry)InnerList[i];

                                if (Entry.id == index_or_ident)
                                {
                                    InnerList[i] = value;
                                }
                            }
                        }
                    }
                }
            }
            public IndexEntry this[string fileName, string type]
            {
                get
                {
                    for (int i = 0; i < InnerList.Count; i++)
                    {
                        if (InnerList[i].GetType() == typeof(IndexEntry))
                        {
                            IndexEntry Entry = (IndexEntry)InnerList[i];

                            if (Entry.FileName == fileName && Entry.Class == type)
                            {
                                return (IndexEntry)InnerList[i];
                            }
                        }
                    }

                    return null;
                }
                set
                {
                    for (int i = 0; i < InnerList.Count; i++)
                    {
                        if (InnerList[i].GetType() == typeof(IndexEntry))
                        {
                            IndexEntry Entry = (IndexEntry)InnerList[i];

                            if (Entry.FileName == fileName && Entry.Class == type)
                            {
                                InnerList[i] = value;
                            }
                        }
                    }
                }
            }

            int IList.Add(object value)
            {
                return InnerList.Add(value);
            }
            public void Add(IndexEntry Entry)
            {
                InnerList.Add(Entry);
            }
            public void AddRange(IndexEntry[] Entries)
            {
                InnerList.AddRange(Entries);
            }

            void IList.Clear()
            {
                InnerList.Clear();
            }
            public void Clear()
            {
                InnerList.Clear();
            }

            bool IList.Contains(object value)
            {
                return InnerList.Contains(value);
            }
            public bool Contains(IndexEntry Entry)
            {
                return InnerList.Contains(Entry);
            }

            IEnumerator IEnumerable.GetEnumerator()
            {
                return InnerList.GetEnumerator();
            }
            public IEnumerator GetEnumerator()
            {
                return InnerList.GetEnumerator();
            }

            int IList.IndexOf(object value)
            {
                return InnerList.IndexOf(value);
            }
            public int IndexOf(IndexEntry Entry)
            {
                return InnerList.IndexOf(Entry);
            }

            void IList.Insert(int index, object value)
            {
                InnerList.Insert(index, value);
            }
            public void Insert(int index, IndexEntry Entry)
            {
                InnerList.Insert(index, Entry);
            }

            bool IList.IsFixedSize
            {
                get { return InnerList.IsFixedSize; }
            }

            void IList.Remove(object value)
            {
                InnerList.Remove(value);
            }
            public bool Remove(IndexEntry Entry)
            {
                if (InnerList.Contains(Entry))
                {
                    InnerList.Remove(Entry);
                    return true;
                }
                else
                    return false;
            }

            void IList.RemoveAt(int index)
            {
                InnerList.RemoveAt(index);
            }
            public void RemoveAt(int index)
            {
                InnerList.RemoveAt(index);
            }

            int ICollection.Count
            {
                get { return InnerList.Count; }
            }
            public int Count
            {
                get { return InnerList.Count; }
            }

            void ICollection.CopyTo(Array array, int index)
            {
                InnerList.CopyTo(array, index);
            }
            bool ICollection.IsSynchronized
            {
                get { return InnerList.IsSynchronized; }
            }
            object ICollection.SyncRoot
            {
                get { return InnerList.SyncRoot; }
            }

            public IndexEntry GetByIdent(int ID)
            {
                for (int i = 0; i < Count; i++)
                {
                    if (InnerList[i].GetType() == typeof(IndexEntry))
                    {
                        if (((IndexEntry)InnerList[i]).id == ID)
                        {
                            return (IndexEntry)InnerList[i];
                        }
                    }
                }

                return null;
            }
        }
    }
}
