﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace AbideHaloLibrary.Map
{
    public class Halo_2_Map : IDisposable
    {
        #region Properties
        public HaloIO IO
        {
            get { return _io; }
        }
        public AbideHaloLibrary.Halo_2_Map.Header MapHeader
        {
            get { return mapHeader; }
        }
        public AbideHaloLibrary.Halo_2_Map.IndexTableHeader IndexTableHeader
        {
            get { return indexHeader; }
        }
        public AbideHaloLibrary.Halo_2_Map.IndexEntry.IndexEntryCollection IndexEntries
        {
            get { return indexEntries; }
        }
        public int MetaStart
        {
            get { return metaStart; }
        }
        public string[] Strings
        {
            get { return stringIDs; }
        }
        public string[] String128s
        {
            get { return string128s; }
        }
        #endregion
        #region Fields
        string file = string.Empty;
        HaloIO _io = null;
        AbideHaloLibrary.Halo_2_Map.Header mapHeader = null;
        AbideHaloLibrary.Halo_2_Map.IndexTableHeader indexHeader = null;
        AbideHaloLibrary.Halo_2_Map.TagListEntry[] tagList = null;
        AbideHaloLibrary.Halo_2_Map.ObjectEntry[] objectList = null;
        AbideHaloLibrary.Halo_2_Map.IndexEntry.IndexEntryCollection indexEntries = null;
        string[] fileList = null;
        string[] stringIDs = null;
        string[] string128s = null;
        int metaStart = 0;
        #endregion
        #region Methods
        public Halo_2_Map(string FileLocation)
        {
            file = FileLocation;
        }
        ~Halo_2_Map()
        {
            Dispose();
        }
        public TreeNode[] GetTreeViewTagTypes()
        {
            //Prepare
            List<TreeNode> Nodes = new List<TreeNode>();
            List<string> Classes = new List<string>();
            TreeNode TempNode = null;

            //Loop...
            for (int i = 0; i < indexEntries.Count; i++)
            {
                //Get Entry...
                AbideHaloLibrary.Halo_2_Map.IndexEntry Entry = indexEntries[i];

                //Add Class Node...
                if (!Classes.Contains(Entry.Class))
                {
                    TempNode = new TreeNode();
                    TempNode.ImageIndex = 1;
                    TempNode.SelectedImageIndex = 1;
                    TempNode.Tag = null;

                    TempNode.Text = string.Format("[{0}] - {1}", Entry.Class.Replace(" ", string.Empty), GetTypeDescription(Entry.Class));
                    Nodes.Add(TempNode);
                    Classes.Add(Entry.Class);
                }

                //Add to appropriate node...
                TempNode = new TreeNode(Entry.FileName);

                //Change Icon?
                HandleClassIcon(ref TempNode, Entry.Class);

                //Final Preparations...
                TempNode.Tag = Entry.Ident;

                //Add...
                Nodes[Classes.IndexOf(Entry.Class)].Nodes.Add(TempNode);
            }

            //Return...
            return Nodes.ToArray();
        }
        public TreeNode[] GetTreeViewFolder()
        {
            //Create List...
            List<TreeNode> Nodes = new List<TreeNode>();
            TreeNode TempNode = null;

            //Loop through Each Tag...
            for (int i = 0; i < indexEntries.Count; i++)
            {
                //Get Entry...
                AbideHaloLibrary.Halo_2_Map.IndexEntry Entry = indexEntries[i];

                //Get Path...
                string[] Path = Entry.FileName.Split('\\');

                if (Path.Length == 1)
                {
                    //Prepare
                    TempNode = new TreeNode(string.Format("{0}.{1}", Path[0], Entry.Class));
                    HandleClassIcon(ref TempNode, Entry.Class);
                    TempNode.Tag = Entry.Ident;

                    //Add...
                    Nodes.Add(TempNode);
                }
                else
                {
                    int Index = 0;
                    bool CreateNew = true;

                    for (int x = 0; x < Nodes.Count; x++)
                    {
                        //Check...
                        if (Nodes[x].Text == Path[0])
                        {
                            Index = x;
                            CreateNew = false;
                            break;
                        }
                    }

                    if (CreateNew)
                    {
                        //Prepare...
                        TempNode = new TreeNode(Path[0]);
                        TempNode.ImageIndex = 1;
                        TempNode.SelectedImageIndex = 1;
                        TempNode.Tag = null;

                        //Loop...
                        LoopTreeViewFolder(TempNode, Entry, 1);

                        //Add...
                        Nodes.Add(TempNode);
                    }
                    else
                    {
                        //Prepare
                        TempNode = Nodes[Index];

                        //Loop...
                        LoopTreeViewFolder(TempNode, Entry, 1);

                        //Add...
                        Nodes[Index] = TempNode;
                    }
                }
            }

            //Return
            return Nodes.ToArray();
        }
        private void LoopTreeViewFolder(TreeNode Parent, AbideHaloLibrary.Halo_2_Map.IndexEntry Entry, int StartingPoint)
        {
            //Prepare...
            TreeNode TempNode = null;
            string[] Path = Entry.FileName.Split('\\');
            bool CreateNew = true;
            int Index = 0;


            //Done?
            if (Path.Length - 1 == StartingPoint)
            {
                //Prepare
                TempNode = new TreeNode(string.Format("{0}.{1}", Path[StartingPoint], Entry.Class));
                HandleClassIcon(ref TempNode, Entry.Class);
                TempNode.Tag = Entry.Ident;

                //Finish...
                Parent.Nodes.Add(TempNode);
                return;
            }
            else
            {
                //Check...
                for (int i = 0; i < Parent.Nodes.Count; i++)
                {
                    if (Parent.Nodes[i].Text == Path[StartingPoint])
                    {
                        CreateNew = false;
                        Index = i;
                        break;
                    }
                }

                //Add / Create...
                if (CreateNew)
                {
                    //Prepare...
                    TempNode = new TreeNode(Path[StartingPoint]);
                    TempNode.ImageIndex = 1;
                    TempNode.SelectedImageIndex = 1;
                    TempNode.Tag = null;

                    //Loop...
                    LoopTreeViewFolder(TempNode, Entry, StartingPoint + 1);

                    //Add...
                    Parent.Nodes.Add(TempNode);
                }
                else
                {
                    //Prepare
                    TempNode = Parent.Nodes[Index];

                    //Loop...
                    LoopTreeViewFolder(TempNode, Entry, StartingPoint + 1);

                    //Add
                    Parent.Nodes[Index] = TempNode;
                }
            }
        }
        private void HandleClassIcon(ref TreeNode Node, string Class)
        {
            //Check Class...
            switch (Class)
            {
                case "bitm":
                    Node.ImageIndex = 3;
                    Node.SelectedImageIndex = 3;
                    break;
                case "snd!":
                    Node.ImageIndex = 4;
                    Node.SelectedImageIndex = 4;
                    break;

                case "sbsp":
                    Node.ImageIndex = 5;
                    Node.SelectedImageIndex = 5;
                    break;

                default:
                    Node.ImageIndex = 2;
                    Node.SelectedImageIndex = 2;
                    break;
            }
        }
        private string GetTypeDescription(string Type)
        {
            string Description = string.Empty;

            switch (Type)
            {
                case "matg":
                    Description = "Match Globals";
                    break;

                case "sncl":
                    Description = "Sound Classes";
                    break;

                case "spk!":
                    Description = "Sound Dialoge";
                    break;

                case "sky ":
                    Description = "Sky";
                    break;

                case "scnr":
                    Description = "Scenario";
                    break;

                case "mode":
                    Description = "Model";
                    break;

                case "shad":
                    Description = "Shader";
                    break;

                case "stem":
                    Description = "Shader Templates";
                    break;

                case "spas":
                    Description = "Shader Passes";
                    break;

                case "bitm":
                    Description = "Bitmap";
                    break;

                case "vrtx":
                    Description = "Vertex Shaders";
                    break;

                case "lens":
                    Description = "Lense Flares";
                    break;

                case "scen":
                    Description = "Scenery Objects";
                    break;

                case "hlmt":
                    Description = "Object Properties";
                    break;

                case "jmad":
                    Description = "JMS Animation Graph";
                    break;

                case "effe":
                    Description = "Effect";
                    break;

                case "prt3":
                    Description = "Particle";
                    break;

                case "pmov":
                    Description = "Particle Physics";
                    break;

                case "lsnd":
                    Description = "Looping Sound";
                    break;

                case "snd!":
                    Description = "Sound";
                    break;

                case "coll":
                    Description = "Collision Geometry";
                    break;

                case "phmo":
                    Description = "Physics Model";
                    break;

                case "PRTM":
                    Description = "Particle Model";
                    break;

                case "ssce":
                    Description = "Sound Scenery";
                    break;

                case "ligh":
                    Description = "Light Object";
                    break;

                case "itmc":
                    Description = "Item Collections";
                    break;

                case "weap":
                    Description = "Weapon";
                    break;

                case "foot":
                    Description = "Material Effect";
                    break;

                case "deca":
                    Description = "Decal";
                    break;

                case "jpt!":
                    Description = "Damage Effect";
                    break;

                case "nhdt":
                    Description = "New HUD Interface";
                    break;

                case "eqip":
                    Description = "Equipment";
                    break;

                case "proj":
                    Description = "Projectile";
                    break;

                case "cont":
                    Description = "Contrail";
                    break;

                case "pphy":
                    Description = "Point Physics";
                    break;

                case "MGS2":
                    Description = "Light Volume";
                    break;

                case "vehc":
                    Description = "Vehicle Collection";
                    break;

                case "vehi":
                    Description = "Vehicle";
                    break;

                case "bloc":
                    Description = "Obstacle";
                    break;

                case "trak":
                    Description = "Camera";
                    break;

                case "ant!":
                    Description = "Antenna";
                    break;

                case "snde":
                    Description = "Environmental Sound";
                    break;

                case "DECR":
                    Description = "Decorator";
                    break;

                case "unic":
                    Description = "Unicode String List";
                    break;
                    
                case "mulg":
                    Description = "Multiplayer Globals";
                    break;

                case "sfx+":
                    Description = "Sound Effects";
                    break;

                case "<fx>":
                    Description = "DSP Effects";
                    break;

                case "snmx":
                    Description = "Sound Mix";
                    break;

                case "adlg":
                    Description = "AI Dialoge";
                    break;

                case "colo":
                    Description = "Color";
                    break;

                case "hudg":
                    Description = "Display Globals";
                    break;

                case "wgtz":
                    Description = "Widget Collections";
                    break;

                case "wigl":
                    Description = "Widget Globals";
                    break;

                case "skin":
                    Description = "UI Skin";
                    break;

                case "wgit":
                    Description = "Widget";
                    break;

                case "bipd":
                    Description = "Biped";
                    break;
                    
                case "garb":
                    Description = "Garbage";
                    break;

                case "udlg":
                    Description = "Unit Dialog";
                    break;

                case "bsdt":
                    Description = "Breakable Surface";
                    break;

                case "gldf":
                    Description = "Global Lighting";
                    break;

                case "clwd":
                    Description = "Blowing Cloth";
                    break;

                case "tdtl":
                    Description = "Beam Trail";
                    break;

                case "char":
                    Description = "Character";
                    break;

                case "styl":
                    Description = "Situational AI Style";
                    break;

                case "crea":
                    Description = "Creature";
                    break;

                case "weat":
                    Description = "Weather";
                    break;

                case "mach":
                    Description = "Machine";
                    break;

                case "mdlg":
                    Description = "Level Specific Dialog";
                    break;

                case "egor":
                    Description = "Screen FX";
                    break;

                case "ctrl":
                    Description = "Control";
                    break;

                case "phys":
                    Description = "Physics";
                    break;

                case "fog ":
                    Description = "Cluster Fog";
                    break;

                case "fpch":
                    Description = "Fog Patch";
                    break;

                case "goof":
                    Description = "Goofy";
                    break;

                case "sily":
                    Description = "Silly Goose";
                    break;

                case "ltmp":
                    Description = "Lightmap";
                    break;

                case "sbsp":
                    Description = "Structure Binary Separation Plane";
                    break;

                case "ugh!":
                    Description = "Sound Diagnostic";
                    break;

                default:
                    Description = "<No Description Set>";
                    Console.WriteLine("Unknown Type: {0}", Type);
                    break;
            }

            return Description;
        }
        public int GetSecondaryMagicOffset(uint RawOffset)
        {
            return (int)(RawOffset - indexHeader.SecondaryMagic);
        }
        public string GetStringIDByID(int ID)
        {
            return stringIDs[ID];
        }
        public string GetSTring128ByID(int ID)
        {
            return string128s[ID];
        }
        public void LoadMap()
        {
            //Check if already Open...
            if (_io != null)
            {
                _io.CloseIO();
                _io.Dispose();
            }

            //Open...
            _io = new HaloIO(file);
            _io.OpenIO();

            //Load Header...
            mapHeader = new AbideHaloLibrary.Halo_2_Map.Header(_io);
            metaStart = mapHeader.IndexOffset + mapHeader.IndexSize;
            indexHeader = new AbideHaloLibrary.Halo_2_Map.IndexTableHeader(mapHeader.IndexOffset, metaStart, _io);

            //Load File List
            _io.MapStream.Position = mapHeader.FileTableOffset;
            string filesString = new string(_io.In.ReadChars(mapHeader.FileTableSize));
            fileList = filesString.Split('\0');
            
            //Load Tag Items
            tagList = new AbideHaloLibrary.Halo_2_Map.TagListEntry[indexHeader.TagListCount];
            for (int i = 0; i < tagList.Length; i++)
            {
                tagList[i] = new AbideHaloLibrary.Halo_2_Map.TagListEntry((mapHeader.IndexOffset + 32) + (i * 12), _io);
            }

            //Load Objects (Idents, Classes, etc...) and Index Entries
            objectList = new AbideHaloLibrary.Halo_2_Map.ObjectEntry[indexHeader.ObjectCount];
            indexEntries = new AbideHaloLibrary.Halo_2_Map.IndexEntry.IndexEntryCollection();
            for (int i = 0; i < objectList.Length; i++)
            {
                objectList[i] = new AbideHaloLibrary.Halo_2_Map.ObjectEntry(indexHeader.ObjectStart + (16 * i), indexHeader.SecondaryMagic, _io);
                indexEntries.Add(new AbideHaloLibrary.Halo_2_Map.IndexEntry(objectList[i], fileList[i]));
            }

            //Load String ID's and String128's
            _io.MapStream.Position = mapHeader.StringTableOffset;
            string[] falseStrings = new string(_io.In.ReadChars(mapHeader.StringTableSize)).Split('\0');
            stringIDs = new string[mapHeader.StringsCount];

            _io.MapStream.Position = mapHeader.Strings128TableOffset;
            string string128sString = new string(_io.In.ReadChars(128 * mapHeader.StringsCount));
            string128s = new string[mapHeader.StringsCount];

            //Loop and Fix Strings
            for (int i = 0; i < mapHeader.StringsCount; i++)
            {
                //String ID's
                stringIDs[i] = falseStrings[i];

                //String 128's
                int offset128 = 128 * i;

                for (int x = 0; x < 128; x++)
                {
                    if (string128sString[offset128 + x] != '\0')
                        string128s[i] += string128sString[offset128 + x];
                }
            }

            //Set First String...
            stringIDs[0] = "Nulled String Reference";
            string128s[0] = "Nulled String Reference";

            //Close Map...
            _io.CloseIO();
            _io.Dispose();
        }
        public void Dispose()
        {
            _io.Dispose();
        }
        #endregion
    }
}
