﻿using System;
using System.IO;

namespace AbideHaloLibrary.Map
{
    public class HaloIO : IDisposable
    {
        /// <summary>
        /// Gets and returns the location of the map file on the HDD.
        /// </summary>
        public string FileLocation
        {
            get { return fileLocation; }
        }
        /// <summary>
        /// Gets and returns a BinaryReader used to read data from the halo map.
        /// </summary>
        public BinaryReader In
        {
            get { return _in; }
        }
        /// <summary>
        /// Gets and returns a BinaryReader used to write data to the halo map.
        /// </summary>
        public BinaryWriter Out
        {
            get { return _out; }
        }
        /// <summary>
        /// Gets and returns a HaloStream reference used to handle the file data's I/O.
        /// </summary>
        public HaloStream MapStream
        {
            get { return _mapStream; }
        }
        /// <summary>
        /// Sets the position to begin reading or writing the map file.
        /// </summary>
        public long Position
        {
            get { return _mapStream.Position; }
            set { _mapStream.Position = value; }
        }

        string fileLocation = string.Empty;
        BinaryReader _in = null;
        BinaryWriter _out = null;
        HaloStream _mapStream = null;

        public HaloIO(string MapLocation)
        {
            if (File.Exists(MapLocation))
            {
                fileLocation = MapLocation;
            }
            else { throw new IOException("Map file does not exist!\r\nFile Location: " + MapLocation); }
        }

        public void Dispose()
        {
            _mapStream.Dispose();
            _in.Dispose();
            _out.Dispose();
        }
        public void OpenIO()
        {
            _mapStream = new HaloStream(fileLocation);
            _in = new BinaryReader(_mapStream);
            _out = new BinaryWriter(_mapStream);
        }
        public void CloseIO()
        {
            _out.Close();
            _in.Close();
            _mapStream.Close();
        }
    }
    public class HaloStream : Stream
    {
        string physicalLocation = string.Empty;
        FileStream innerStream = null;

        public HaloStream(string MapFileLocation)
        {
            physicalLocation = MapFileLocation;
            innerStream = new FileStream(MapFileLocation, FileMode.Open);
        }
        public override bool CanRead
        {
            get { return innerStream.CanRead; }
        }
        public override bool CanSeek
        {
            get { return innerStream.CanSeek; }
        }
        public override bool CanWrite
        {
            get { return innerStream.CanWrite; }
        }
        public override void Flush()
        {
            innerStream.Flush();
        }
        public override long Length
        {
            get { return innerStream.Length; }
        }
        public override long Position
        {
            get
            {
                return innerStream.Position;
            }
            set
            {
                innerStream.Position = value;
            }
        }
        public override int Read(byte[] buffer, int offset, int count)
        {
            return innerStream.Read(buffer, offset, count);
        }
        public override long Seek(long offset, SeekOrigin origin)
        {
            return innerStream.Seek(offset, origin);
        }
        public override void SetLength(long value)
        {
            innerStream.SetLength(value);
        }
        public override void Write(byte[] buffer, int offset, int count)
        {
            innerStream.Write(buffer, offset, count);
        }
        public override void Close()
        {
            base.Close();
            innerStream.Close();
        }
        protected override void Dispose(bool disposing)
        {
            innerStream.Close();
            innerStream.Dispose();
            base.Dispose(disposing);
        }
    }
}
