﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Xml;

namespace AbideHaloLibrary.Meta
{
    public class HaloPlugin
    {
        public enum PluginErrorType
        {
            MissingAttribute,
            EmptyAttribute,
            SameValueEntry
        }

        public HaloMetaObject MainObject;
        public string FileName;
        public List<HaloMetaObject> Revisions;
        public List<PluginErrorType> PluginErrors;

        private XmlDocument PluginDocument;

        public HaloPlugin()
        {
            MainObject = new HaloMetaObject();

            MainObject.ChildObjects = new List<HaloMetaObject>();

            PluginErrors = new List<PluginErrorType>();
        }

        public HaloPlugin(string PluginFileName) : this()
        {
            FileName = PluginFileName;
        }

        public bool Load(string PluginFileName)
        {
            FileName = PluginFileName;

            return Load();
        }

        public bool Load()
        {
            //Check If Plugin Exists
            if (!File.Exists(FileName))
            {
                return false;
            }

            Revisions = new List<HaloMetaObject>();
            PluginDocument = new XmlDocument();

            //Load Document
            PluginDocument.Load(FileName);

            //Get Plugin Information
            MainObject.Class = GetAttribute(PluginDocument.FirstChild, "class");
            MainObject.Author = GetAttribute(PluginDocument.FirstChild, "author");
            MainObject.Version = GetAttribute(PluginDocument.FirstChild, "version");
            MainObject.Size = GetAttribute(PluginDocument.FirstChild, "headersize");

            if (MainObject.Class == "null")
            {
                MainObject.Class = GetAttribute(PluginDocument.FirstChild, "name");
                MainObject.Size = GetAttribute(PluginDocument.FirstChild, "size");
            }

            //Start Loading Child Nodes
            MainObject.ChildObjects.AddRange(LoadChildNode(PluginDocument.FirstChild));

            return true;
        }

        private List<HaloMetaObject> LoadChildNode(XmlNode ParentNode)
        {
            HaloMetaObject MetaObject;
            List<HaloMetaObject> ChildObjects = new List<HaloMetaObject>();

            //Loop through all child nodes
            foreach (XmlNode ChildNode in ParentNode.ChildNodes)
            {
                MetaObject = new HaloMetaObject();

                //Get Basic Object Information
                MetaObject.Type = ChildNode.Name.ToLower();
                MetaObject.Name = GetAttribute(ChildNode, "name");
                MetaObject.Offset = GetAttribute(ChildNode, "offset");
                MetaObject.Visible = GetAttribute(ChildNode, "visible");
                MetaObject.Size = GetAttribute(ChildNode, "size");
                MetaObject.Label = GetAttribute(ChildNode, "label");

                //Get Block Index Information If Possible
                MetaObject.ReflexiveOffset = GetAttribute(ChildNode, "reflexiveoffset");
                MetaObject.ReflexiveSize = GetAttribute(ChildNode, "reflexivesize");
                MetaObject.ItemOffset = GetAttribute(ChildNode, "itemoffset");
                MetaObject.ItemType = GetAttribute(ChildNode, "itemtype");
                MetaObject.Layer = GetAttribute(ChildNode, "layer");

                //Get Other Object Information
                switch (MetaObject.Type)
                {
                    case "revision":
                        //Load Revision Information
                        MetaObject.Author = GetAttribute(ChildNode, "author");
                        MetaObject.Version = GetAttribute(ChildNode, "version");
                        MetaObject.Description = ChildNode.InnerText;

                        //Add To Revisions
                        Revisions.Add(MetaObject);
                        break;

                    default:
                        ChildObjects.Add(MetaObject);
                        break;
                }

                //Load Child Nodes
                if (ChildNode.ChildNodes.Count > 0)
                {
                    MetaObject.ChildObjects = new List<HaloMetaObject>();

                    MetaObject.ChildObjects.AddRange(LoadChildNode(ChildNode));
                }
            }

            return ChildObjects;
        }

        public string GetAttribute(XmlNode Node, string AttributeName)
        {
            string AttributeValue = "null";

            //Get attribute if possible
            if (Node != null && Node.Attributes != null && Node.Attributes[AttributeName] != null)
            {
                AttributeValue = Node.Attributes[AttributeName].Value;
            }

            return AttributeValue;
        }
    }
}
