﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Windows.Forms.Design;

namespace Potential_Software.Potential_UI.Forms
{
    public class CaptionButton : Control
    {
        /// <summary>
        /// Normal Image
        /// </summary>
        [Category("Appearance"), Description("The image to appear over the control when the control is not being interacted with.")]
        public Image Normal
        {
            get { return norm; }
            set { norm = value; this.Size = GetSize(); if (DesignMode) { this.BackgroundImage = value; } }
        }
        /// <summary>
        /// Hot Image
        /// </summary>
        [Category("Appearance"), Description("The image to appear over the control when the mouse enters the control.")]
        public Image Hot
        {
            get { return hot; }
            set { hot = value; this.Size = GetSize(); }
        }
        /// <summary>
        /// Pressed Image
        /// </summary>
        [Category("Appearance"), Description("The image to appear over the control when the mouse clickes the control.")]
        public Image Pressed
        {
            get { return down; }
            set { down = value; this.Size = GetSize(); }
        }
        /// <summary>
        /// Disabled Image
        /// </summary>
        [Category("Appearance"), Description("The image to appear over the control when the control is disabled.")]
        public Image Disabled
        {
            get { return disable; }
            set { disable = value; this.Size = GetSize(); }
        }
        /// <summary>
        /// Tells the button to draw disabled or enabled image
        /// </summary>
        [Browsable(false)]
        public bool Active
        {
            get { return active; }
            set { active = value; Refresh(); BackgroundImage = value == true ? Normal : Disabled; }
        }

        private Image norm;
        private Image hot;
        private Image down;
        private Image disable;
        private bool active;

        public CaptionButton()
        {
            norm = (Image)SystemIcons.Application.ToBitmap();
            hot = (Image)SystemIcons.Application.ToBitmap();
            down = (Image)SystemIcons.Application.ToBitmap();
            disable = (Image)SystemIcons.Application.ToBitmap();
            
            SetStyle(ControlStyles.SupportsTransparentBackColor, true);
            BackgroundImage = Normal;
            BackgroundImageLayout = ImageLayout.Center;
            BackColor = Color.Transparent;
        }
        protected override void SetBoundsCore(int x, int y, int width, int height, BoundsSpecified specified)
        {
            base.SetBoundsCore(x, y, GetSize().Width, GetSize().Height, specified);
        }
        protected override void OnMouseEnter(EventArgs e)
        {
            BackgroundImage = Hot;
            base.OnMouseEnter(e);
        }
        protected override void OnMouseLeave(EventArgs e)
        {
            if (Active)
                BackgroundImage = Normal;
            else
                BackgroundImage = Disabled;

            base.OnMouseLeave(e);
        }
        protected override void OnMouseDown(MouseEventArgs e)
        {
            this.BackgroundImage = Pressed;
            base.OnMouseDown(e);
        }
        protected override void OnMouseUp(MouseEventArgs e)
        {
            this.BackgroundImage = Hot;
            base.OnMouseUp(e);
        }
        private Size GetSize()
        {
            int normW = 0;
            int hotW = 0;
            int downW = 0;
            int disW = 0;
            int normH = 0;
            int hotH = 0;
            int downH = 0;
            int disH = 0;

            if (norm != null)
            { normW = norm.Width; normH = norm.Height; }
            if (hot != null)
            { hotW = hot.Width; hotH = hot.Height; }
            if (down != null)
            { downW = down.Width; downH = down.Height; }
            if (disable != null)
            { disW = disable.Width; disH = disable.Height; }

            return new Size(Math.Max(normW, Math.Max(hotW, Math.Max(downW, disW))), Math.Max(normH, Math.Max(hotH, Math.Max(downH, disH))));
        }
    }
}
