﻿using Potential_Software.Win32;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Potential_Software.Potential_UI.Forms.FX
{
    public class LayeredFxForm : Form
    {
        protected override CreateParams CreateParams
        {
            get
            {
                if (DesignMode)
                    return base.CreateParams;

                CreateParams Params = base.CreateParams;

                Params.Style = (int)(WinUser.WS_POPUP | WinUser.WS_CLIPSIBLINGS | WinUser.WS_CLIPCHILDREN);
                Params.ExStyle = (int)(WinUser.WS_EX_LEFT | WinUser.WS_EX_LTRREADING | WinUser.WS_EX_RIGHTSCROLLBAR | WinUser.WS_EX_TOOLWINDOW | WinUser.WS_EX_LAYERED);
                return Params;
            }
        }

        public LayeredFxForm()
        {
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
        }
        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);

            Bitmap FormBitm = new Bitmap(Width, Height);

            using (Graphics imgGfx = Graphics.FromImage(FormBitm))
            {
                using (PaintEventArgs pe = new PaintEventArgs(imgGfx, new Rectangle(0, 0, Width, Height)))
                {
                    OnPaintLayeredWindow(pe);
                }
            }

            IntPtr screenDc = User32.GetDC(IntPtr.Zero);
            IntPtr memDc = Gdi32.CreateCompatibleDC(screenDc);
            IntPtr hBitmap = IntPtr.Zero;
            IntPtr oldBitmap = IntPtr.Zero;

            try
            {
                hBitmap = FormBitm.GetHbitmap(Color.FromArgb(0x0));
                oldBitmap = Gdi32.SelectObject(memDc, hBitmap);

                SIZE size = new SIZE(); size.cx = FormBitm.Width; size.cy = FormBitm.Height;
                POINT pointSource = new POINT(); pointSource.x = 0; pointSource.y = 0;
                POINT topPos = new POINT(); topPos.x = Left; topPos.y = Top;

                BLENDFUNCTION blend = new BLENDFUNCTION();
                blend.BlendOp = WinUser.AC_SRC_OVER;
                blend.BlendFlags = 0x0;
                blend.SourceConstantAlpha = 0xFF;
                blend.AlphaFormat = WinUser.AC_SRC_ALPHA;

                User32.UpdateLayeredWindow(this.Handle, screenDc, ref topPos, ref size, memDc, ref pointSource, 0, ref blend, WinUser.ULW_ALPHA);
            }
            finally
            {
                User32.ReleaseDC(IntPtr.Zero, screenDc);

                if (hBitmap != IntPtr.Zero)
                {
                    Gdi32.SelectObject(memDc, oldBitmap);
                    Gdi32.DeleteObject(hBitmap);
                }

                Gdi32.DeleteDC(memDc);
                FormBitm.Dispose();
            }
        }
        protected override void WndProc(ref Message m)
        {
            if (DesignMode)
            {
                base.WndProc(ref m);
                return;
            }

            Messages msg = (Messages)m.Msg;
            switch (msg)
            {
                case Messages.WM_NCHITTEST:
                    m.Result = OnNonClientHitTest(PointToClient(new Point(m.LParam.ToInt32())));
                    break;

                default:
                    base.WndProc(ref m);
                    break;
            }
        }
        protected virtual IntPtr OnNonClientHitTest(Point ClientPoint)
        {
            return (IntPtr)WinUser.HTCLIENT;
        }
        protected virtual void OnPaintLayeredWindow(PaintEventArgs e)
        {
        }
    }
}
