﻿using Potential_Software.Win32;
using System;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Text;

namespace Potential_Software.Potential_UI.Forms
{
    public class NativeWindowTest : IDisposable
    {
        readonly int MAX_LOADSTING = 100;
        readonly IntPtr NULL = (IntPtr)0;
        readonly IntPtr FALSE = (IntPtr)0;
        readonly IntPtr TRUE = (IntPtr)1;

        IntPtr hwnd = IntPtr.Zero;
        IntPtr hInst = IntPtr.Zero;
        string szTitle = string.Empty; StringBuilder sbTitle = null;
        string szWindowClass = string.Empty; StringBuilder sbWindowClass = null;

        public NativeWindowTest(string Title, string Class)
        {
            //Set Title and Class
            szTitle = Title;
            szWindowClass = Class;

            //Check WindowClass
            if (string.IsNullOrEmpty(Class))
            {
                throw new NotImplementedException("Window class cannot be nulled or empty!");
            }

            //Setup StringBuilders
            sbTitle = new StringBuilder(Title, MAX_LOADSTING);
            sbWindowClass = new StringBuilder(Class, MAX_LOADSTING);

            //Get HINSTANCE...
            IntPtr hInstance = Kernel32.GetModuleHandle(null);
            _tWinMain(hInstance, NULL, string.Empty, 10);
        }
        public int _tWinMain(IntPtr hInstance, IntPtr hPrevInstance, string lpCmdLine, int nCmdShow)
        {
            MSG msg = new MSG();
            IntPtr hAccelTable = NULL;

            MyRegisterClass(hInstance);

            //Create Instance
            bool Initialized = InitInstance(hInstance, nCmdShow);
            if (!Initialized)
            {
                return 0;
            }

            hAccelTable = User32.LoadAccelerators(hInstance, WinUser.MAKEINTRESOURCE(109));

            //Message Loop
            while (User32.GetMessage(ref msg, NULL, 0, 0) == TRUE)
            {
                User32.SendMessage(msg.hwnd, msg.message, msg.wParam.ToInt32(), msg.lParam.ToInt32());
            }

            return (int)msg.wParam;
        }
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }
        public void Dispose(bool Disposing)
        {
            if (hwnd != NULL && Disposing)
            {
                User32.DestroyWindow(hwnd);
                hwnd = NULL;
            }
        }
        ushort MyRegisterClass(IntPtr hInstance)
        {
            WNDCLASSEX wcex = new WNDCLASSEX();

            wcex.cbSize = (uint)Marshal.SizeOf(typeof(WNDCLASSEX));
            wcex.style = WinUser.CS_HREDRAW | WinUser.CS_VREDRAW;
            wcex.lpfnWndProc = WndProc;
            wcex.cbClsExtra = 0;
            wcex.cbWndExtra = 0;
            wcex.hInstance = hInstance;
            wcex.hIcon = NULL;
            wcex.hCursor = NULL;
            wcex.hbrBackground = NULL;
            wcex.lpszMenuName = "FormMenu";
            wcex.lpszClassName = szWindowClass;
            wcex.hIconSm = NULL;

            return User32.RegisterClassEx(ref wcex);
        }
        bool InitInstance(IntPtr hInstance, int nCmdShow)
        {
            hwnd = NULL;

            hInst = hInstance;

            hwnd = User32.CreateWindowEx(0, sbWindowClass, sbTitle, (uint)WinUser.WS_OVERLAPPEDWINDOW,
                0, 0, 300, 300, NULL, NULL, hInstance, NULL);

            if (hwnd == FALSE)
            {
                return false;
            }

            User32.ShowWindow(hwnd, nCmdShow);
            User32.UpdateWindow(hwnd);

            return true;
        }
        protected virtual IntPtr WndProc(IntPtr hWnd, uint message, IntPtr wParam, IntPtr lParam)
        {
            IntPtr LRESULT = NULL;
            Messages msg = (Messages)message;
            IntPtr hDC = NULL;
            Graphics Gfx = null;

            switch (msg)
            {
                case Messages.WM_PAINT:
                    hDC = User32.GetDC(hWnd);

                    using (Gfx = Graphics.FromHdc(hDC))
                    {
                        Gfx.Clear(SystemColors.Control);
                    }

                    LRESULT = (IntPtr)0;
                    break;
                default:
                    LRESULT = User32.DefWindowProc(hWnd, message, wParam, lParam);
                    break;
            }

            return LRESULT;
        }
    }
}
